﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;

namespace HeuristicLab.Problems.Robocode {
  public class Interpreter {

    static string chars = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";

    public static double EvaluateTankProgram(ISymbolicExpressionTree tree, string path) {
      //TankStats tankStats = EvaluateTankProgram(tree.Root, solutionCandidateRoot, null);
      //Evaluator.tankStatLog += "\r\n" + tankStats.shots + " / " + tankStats.moves;
      string interpretedProgram = InterpretProgramTree(tree.Root);
      Random random = new Random();
      string formattedPath = path.Replace("/", "\\");
      string outputname = "";
      string output = "";
      bool noError = true;
      do {
        try {
          outputname = new string(Enumerable.Repeat(chars, 8).Select(s => s[random.Next(s.Length)]).ToArray());
          output = /*solutionCandidateRoot.InnerText*/interpretedProgram.Replace("class output", "class " + outputname);
          File.WriteAllText(path + "/robots/Evaluation/" + outputname + ".java", output, System.Text.Encoding.Default);
          noError = true;
        }
        catch (Exception e) { noError = false; }
      } while (!noError);

      //string path = System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location);
      //path.Replace("\\", "\\\\");
      //path.Replace(" ", "\\ ");
      #region Compile code file

      ProcessStartInfo javaCompileInfo = new ProcessStartInfo();
      javaCompileInfo.FileName = "cmd.exe";
      javaCompileInfo.Arguments = "/C javac -cp " + formattedPath + "\\libs\\robocode.jar " + formattedPath + "\\robots\\Evaluation\\" + outputname + ".java";
      javaCompileInfo.RedirectStandardOutput = true;
      javaCompileInfo.RedirectStandardError = true;
      javaCompileInfo.UseShellExecute = false;
      javaCompileInfo.CreateNoWindow = true;

      Process javaCompile = new Process();
      javaCompile.StartInfo = javaCompileInfo;
      javaCompile.Start();

      /*Evaluator.tankStatLog = "=====================================================\r\n";
      Evaluator.tankStatLog += "Tree Compilation Output: \r\n";
      Evaluator.tankStatLog += "=====================================================\r\n";*/

      string cmdOutput = javaCompile.StandardOutput.ReadToEnd();
      cmdOutput += javaCompile.StandardError.ReadToEnd();

      //Evaluator.tankStatLog += cmdOutput;
      //Console.WriteLine(cmdOutput);
      //Console.ReadLine();

      /*Evaluator.tankStatLog = "=====================================================\r\n";
      Evaluator.tankStatLog += "End of Tree Compilation Output: \r\n";
      Evaluator.tankStatLog += "=====================================================\r\n";*/

      javaCompile.WaitForExit();
      if (javaCompile.ExitCode != 0) {
        return -1000.0;
      }
      #endregion


      try { File.Delete(path + "/robots/robot.database"); }
      catch (Exception e) { }

      #region evaluate code

      ProcessStartInfo evaluateCodeInfo = new ProcessStartInfo();
      evaluateCodeInfo.FileName = "cmd.exe";
      //javaCompileInfo.Arguments = "/C javac -cp C:\\robocode\\libs\\robocode.jar \"" + path + "\\Spaced Up\\output.java\"";
      evaluateCodeInfo.Arguments = "/C java -classpath " + formattedPath + "\\libs;" + formattedPath + "\\libs\\robocode.core-1.8.1.0.jar;" + formattedPath + "\\libs\\robocode.jar;" + formattedPath + "\\libs\\picocontainer-2.14.2.jar BattleRunner Evaluation." + outputname + "* false";
      //Console.WriteLine(javaCompileInfo.Arguments);
      evaluateCodeInfo.RedirectStandardOutput = true;
      evaluateCodeInfo.RedirectStandardError = true;
      evaluateCodeInfo.UseShellExecute = false;
      evaluateCodeInfo.CreateNoWindow = true;

      Process evaluateCode = new Process();
      evaluateCode.StartInfo = evaluateCodeInfo;
      evaluateCode.Start();

      StringBuilder q = new StringBuilder();
      string scoreString = "";
      evaluateCode.WaitForExit();

      try {
        scoreString = evaluateCode.StandardOutput.ReadToEnd().Split(
        new string[] { "\r\n" }, StringSplitOptions.RemoveEmptyEntries).Last();
        q.Append("Output: " + scoreString + "\r\n");
        q.Append("Error: " + evaluateCode.StandardError.ReadToEnd() + "\r\n");
        cmdOutput += q.ToString();
      }
      catch {
        return -1000.0;
      }

      #endregion

      double evaluation = -900;
      try {
        File.Delete(path + "/robots/Evaluation/" + outputname + ".java");
        File.Delete(path + "/robots/Evaluation/" + outputname + ".class");
        evaluation = Double.Parse(scoreString);
      }
      catch (Exception e) { evaluation = -900; }
      Random rand = new Random();
      evaluation += rand.NextDouble() / 100;

      return evaluation;
    }

    public static string InterpretProgramTree(ISymbolicExpressionTreeNode node) {
      var tankNode = node;
      while (!(tankNode.Symbol is Tank))
        tankNode = tankNode.GetSubtree(0);
      return ((CodeNode)tankNode.Symbol).Interpret(tankNode, tankNode.Subtrees);
    }
  }
}